package com.persagy.apm.energy.report.monthly.detail.business.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.detail.business.dao.ReportBusinessDetailMapper;
import com.persagy.apm.energy.report.monthly.detail.business.service.IReportBusinessDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.detail.business.model.*;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告详情(ReportBusinessDetail) service层
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:08:34
 */
@Service
public class ReportBusinessDetailServiceImpl extends ServiceImpl<ReportBusinessDetailMapper, ReportBusinessDetail>
        implements IReportBusinessDetailService {

    /**
     * 创建报告详情
     *
     * @return 报告详情主键
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public String createReportBusinessDetail(AddReportBusinessDetailDTO addReportBusinessDetailDTO) {
        ReportBusinessDetail reportBusinessDetail = ConvertReportBusinessDetailTool.INSTANCE.convertAddDto2Entity(addReportBusinessDetailDTO);
        // 设置默认值
        setDefaultValue(reportBusinessDetail);
        save(reportBusinessDetail);
        return reportBusinessDetail.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportBusinessDetail 报告详情实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportBusinessDetail reportBusinessDetail) {
        reportBusinessDetail.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告详情详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public ReportBusinessDetail queryReportBusinessDetailDetail(String id) {
        ReportBusinessDetail reportBusinessDetail = getById(id);
        if (reportBusinessDetail == null) {
            throw new IllegalArgumentException("查看ReportBusinessDetail详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportBusinessDetail;
    }

    /**
     * 更新报告详情
     *
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public void updateReportBusinessDetail(UpdateReportBusinessDetailDTO updateReportBusinessDetailDTO) {
        ReportBusinessDetail reportBusinessDetail = getById(updateReportBusinessDetailDTO.getId());
        reportBusinessDetail = ConvertReportBusinessDetailTool.INSTANCE.convertUpdateDto2Entity(reportBusinessDetail, updateReportBusinessDetailDTO);
        reportBusinessDetail.setModifier(AppContext.getContext().getAccountId());
        updateById(reportBusinessDetail);
    }

    /**
     * 校验报告详情是否可删除
     *
     * @param id 报告详情主键
     * @return 报告详情do类
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    public ReportBusinessDetail checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportBusinessDetail时发生异常，主键为空");
        }

        ReportBusinessDetail reportBusinessDetail = getById(id);

        if (reportBusinessDetail == null) {
            throw new IllegalArgumentException("删除ReportBusinessDetail时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportBusinessDetail;
    }

    /**
     * 删除报告详情
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public void deleteReportBusinessDetail(String id) {
        // 校验是否可删除
        ReportBusinessDetail reportBusinessDetail = checkDeletable(id);

        reportBusinessDetail.setValid(ValidEnum.FALSE.getType());
        updateById(reportBusinessDetail);
    }

    /**
     * 查询报告详情
     *
     * @return List<ReportBusinessDetail>
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public List<ReportBusinessDetail> queryReportBusinessDetailList(QueryReportBusinessDetailDTO queryReportBusinessDetailDTO) {
        QueryWrapper<ReportBusinessDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportBusinessDetail.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportBusinessDetail.PROP_CREATIONTIME);

        if (queryReportBusinessDetailDTO != null) {

            if (queryReportBusinessDetailDTO.getOpenPower() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_POWER, queryReportBusinessDetailDTO.getOpenPower());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getOpenPowerExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_OPEN_POWER_EXPLAIN, queryReportBusinessDetailDTO.getOpenPowerExplain());
            }

            if (queryReportBusinessDetailDTO.getComparePower() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_POWER, queryReportBusinessDetailDTO.getComparePower());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getComparePowerExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_POWER_EXPLAIN, queryReportBusinessDetailDTO.getComparePowerExplain());
            }

            if (queryReportBusinessDetailDTO.getOpenCost() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_COST, queryReportBusinessDetailDTO.getOpenCost());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getOpenCostExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_OPEN_COST_EXPLAIN, queryReportBusinessDetailDTO.getOpenCostExplain());
            }

            if (queryReportBusinessDetailDTO.getCompareCost() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_COST, queryReportBusinessDetailDTO.getCompareCost());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getCompareCostExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_COST_EXPLAIN, queryReportBusinessDetailDTO.getCompareCostExplain());
            }

            if (queryReportBusinessDetailDTO.getJob() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_JOB, queryReportBusinessDetailDTO.getJob());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getRemark())) {
                queryWrapper.like(ReportBusinessDetail.PROP_REMARK, queryReportBusinessDetailDTO.getRemark());
            }

            if (queryReportBusinessDetailDTO.getPlatform() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_PLATFORM, queryReportBusinessDetailDTO.getPlatform());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getItemExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_ITEM_EXPLAIN, queryReportBusinessDetailDTO.getItemExplain());
            }

            if (queryReportBusinessDetailDTO.getProjectIds() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_PROJECT_IDS, queryReportBusinessDetailDTO.getProjectIds());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告详情
     *
     * @return IPage<ReportBusinessDetail>
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public IPage<ReportBusinessDetail> pageQueryReportBusinessDetail(PageQueryReportBusinessDetailDTO pageQueryReportBusinessDetailDTO) {
        QueryWrapper<ReportBusinessDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportBusinessDetail.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportBusinessDetail> pageParam = new Page<>(pageQueryReportBusinessDetailDTO.getPage(), pageQueryReportBusinessDetailDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportBusinessDetailDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportBusinessDetail.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportBusinessDetailDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (pageQueryReportBusinessDetailDTO.getOpenPower() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_POWER, pageQueryReportBusinessDetailDTO.getOpenPower());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getOpenPowerExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_OPEN_POWER_EXPLAIN, pageQueryReportBusinessDetailDTO.getOpenPowerExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getComparePower() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_POWER, pageQueryReportBusinessDetailDTO.getComparePower());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getComparePowerExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_POWER_EXPLAIN, pageQueryReportBusinessDetailDTO.getComparePowerExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getOpenCost() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_COST, pageQueryReportBusinessDetailDTO.getOpenCost());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getOpenCostExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_OPEN_COST_EXPLAIN, pageQueryReportBusinessDetailDTO.getOpenCostExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getCompareCost() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_COST, pageQueryReportBusinessDetailDTO.getCompareCost());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getCompareCostExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_COST_EXPLAIN, pageQueryReportBusinessDetailDTO.getCompareCostExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getJob() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_JOB, pageQueryReportBusinessDetailDTO.getJob());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getRemark())) {
            queryWrapper.like(ReportBusinessDetail.PROP_REMARK, pageQueryReportBusinessDetailDTO.getRemark());
        }

        if (pageQueryReportBusinessDetailDTO.getPlatform() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_PLATFORM, pageQueryReportBusinessDetailDTO.getPlatform());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getItemExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_ITEM_EXPLAIN, pageQueryReportBusinessDetailDTO.getItemExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getProjectIds() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_PROJECT_IDS, pageQueryReportBusinessDetailDTO.getProjectIds());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
