package com.persagy.apm.energy.report.monthly.outline.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.outline.dao.ReportOutlineMapper;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.outline.model.*;
import com.persagy.apm.energy.report.monthly.outline.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告概要(ReportOutline) service层
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:09:37
 */
@Service
public class ReportOutlineServiceImpl extends ServiceImpl<ReportOutlineMapper, ReportOutline>
        implements IReportOutlineService {

    /**
     * 创建报告概要
     *
     * @return 报告概要主键
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public String createReportOutline(AddReportOutlineDTO addReportOutlineDTO) {
        ReportOutline reportOutline = ConvertReportOutlineTool.INSTANCE.convertAddDto2Entity(addReportOutlineDTO);
        // 设置默认值
        setDefaultValue(reportOutline);
        save(reportOutline);
        return reportOutline.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportOutline 报告概要实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportOutline reportOutline) {
        reportOutline.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告概要详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public ReportOutline queryReportOutlineDetail(String id) {
        ReportOutline reportOutline = getById(id);
        if (reportOutline == null) {
            throw new IllegalArgumentException("查看ReportOutline详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportOutline;
    }

    /**
     * 更新报告概要
     *
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public void updateReportOutline(UpdateReportOutlineDTO updateReportOutlineDTO) {
        ReportOutline reportOutline = getById(updateReportOutlineDTO.getId());
        reportOutline = ConvertReportOutlineTool.INSTANCE.convertUpdateDto2Entity(reportOutline, updateReportOutlineDTO);
        reportOutline.setModifier(AppContext.getContext().getAccountId());
        updateById(reportOutline);
    }

    /**
     * 校验报告概要是否可删除
     *
     * @param id 报告概要主键
     * @return 报告概要do类
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    public ReportOutline checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportOutline时发生异常，主键为空");
        }

        ReportOutline reportOutline = getById(id);

        if (reportOutline == null) {
            throw new IllegalArgumentException("删除ReportOutline时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportOutline;
    }

    /**
     * 删除报告概要
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public void deleteReportOutline(String id) {
        // 校验是否可删除
        ReportOutline reportOutline = checkDeletable(id);

        reportOutline.setValid(ValidEnum.FALSE.getType());
        updateById(reportOutline);
    }

    /**
     * 查询报告概要
     *
     * @return List<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public List<ReportOutline> queryReportOutlineList(QueryReportOutlineDTO queryReportOutlineDTO) {
        QueryWrapper<ReportOutline> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportOutline.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportOutline.PROP_CREATIONTIME);

        if (queryReportOutlineDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getName())) {
                queryWrapper.like(ReportOutline.PROP_NAME, queryReportOutlineDTO.getName());
            }

            if (queryReportOutlineDTO.getDate() != null) {
                queryWrapper.eq(ReportOutline.PROP_DATE, queryReportOutlineDTO.getDate());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getBelong())) {
                queryWrapper.like(ReportOutline.PROP_BELONG, queryReportOutlineDTO.getBelong());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getType())) {
                queryWrapper.like(ReportOutline.PROP_TYPE, queryReportOutlineDTO.getType());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getDetailId())) {
                queryWrapper.like(ReportOutline.PROP_DETAIL_ID, queryReportOutlineDTO.getDetailId());
            }

            if (queryReportOutlineDTO.getStatus() != null) {
                queryWrapper.eq(ReportOutline.PROP_STATUS, queryReportOutlineDTO.getStatus());
            }

            if (queryReportOutlineDTO.getRemark() != null) {
                queryWrapper.eq(ReportOutline.PROP_REMARK, queryReportOutlineDTO.getRemark());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告概要
     *
     * @return IPage<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public IPage<ReportOutline> pageQueryReportOutline(PageQueryReportOutlineDTO pageQueryReportOutlineDTO) {
        QueryWrapper<ReportOutline> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportOutline.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportOutline> pageParam = new Page<>(pageQueryReportOutlineDTO.getPage(), pageQueryReportOutlineDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportOutlineDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportOutline.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportOutlineDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportOutlineDTO.getName())) {
            queryWrapper.like(ReportOutline.PROP_NAME, pageQueryReportOutlineDTO.getName());
        }

        if (pageQueryReportOutlineDTO.getStartDate() != null) {
            queryWrapper.gt(ReportOutline.PROP_DATE, pageQueryReportOutlineDTO.getStartDate());
        }

        if (pageQueryReportOutlineDTO.getEndDate() != null) {
            queryWrapper.lt(ReportOutline.PROP_DATE, pageQueryReportOutlineDTO.getEndDate());
        }

        if (pageQueryReportOutlineDTO.getStatus() != null) {
            queryWrapper.eq(ReportOutline.PROP_STATUS, pageQueryReportOutlineDTO.getStatus());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
